/*
 *  radiusplugin -- An OpenVPN plugin for do radius authentication 
 *					and accounting.
 * 
 *  Copyright (C) 2005 EWE TEL GmbH/Ralf Luebben <ralfluebben@gmx.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */


#include "Config.h"


/** The constructor The constructor initializes all char arrays with 0.
 */

Config::Config(void)
{
	
	
	memset(this->ccdPath,0,256);
	memset(this->statusFile,0,256);
	memset(this->subnet,0,16);
	memset(this->p2p,0,16);
}

/** The constructor initializes all char arrays with 0. After the initialization
 * the configfile is parsed and the information which are
 * found are copied to the attributes.
 * @param configfile The name of the configfile.
 */

Config::Config(char * configfile)
{
	memset(this->ccdPath,0,256);
	memset(this->statusFile,0,256);
	memset(this->subnet,0,16);
	memset(this->p2p,0,16);
	this->parseConfigFile(configfile);
}


/** The destructur clears the serverlist. */
Config::~Config(void)
{
	
	
}



/** The method parse the configfile for attributes and 
 * radius server, the attributes are copied to the
 * member variables.
 * @param configfile The name of the configfile.
 * @return An integer, 0 if everything is ok 
 * or PARSING_ERROR or BAD_FILE if something is wrong.*/
int Config::parseConfigFile(const char * configfile)
{
	char line[256];
	char tmp[50];

	
	ifstream file;
	file.open(configfile, ios::in);
	if (file.is_open())
	{
		while (file.eof()==false)
		{
			file.getline(line, 255);
			line[256]='\0';
			this->deletechars(line);
			
			if (strstr(line,"ccdPath"))
			{
				this->getValue(line, tmp);
				strncpy(this->ccdPath,tmp,256);
			}
			if (strstr(line,"statusFile"))
			{
				this->getValue(line, tmp);
				strncpy(this->statusFile,tmp,256);
			}
			if (strstr(line,"subnet"))
			{
				this->getValue(line, tmp);
				strncpy(this->subnet,tmp,16);
			}
			if (strstr(line,"p2p"))
			{
				this->getValue(line, tmp);
				strncpy(this->p2p,tmp,16);
			}
			
		}
		file.close();
	}
	else
	{
		return BAD_FILE;
	}
	return 0;
}
	
	
/** The method deletes chars from a string.
 * This is used to delete tabs, spaces, # and '\0'
 * from a string.
 * @param text The string which should be cleaned.
 */ 
void Config::deletechars(char * text)
{
	int i=0, len, j=0;
	char * newtext;
	len=strlen(text);
	newtext=new char[len+1];
	
	while(i<=len)
	{
		if(text[i]!=' ' && text[i]!='\t' && text[i]!='#' && text[i]!='\0')
		{
			newtext[j]=text[i];
			j++;
		}
		i++;
	}
	newtext[j]='\0';
	strcpy(text, newtext);
	delete [] (newtext);
}


/**The method finds the part of the string after the
 * '=' and puts it in the value.
 * @param text The string with the value.
 * @param value The value where to put the part of the string. */
void Config::getValue(char * text, char * value)
{
	
	int i=0,j=0;
	while (text[i]!='=' && text[i]!='\0')
	{
		i++;
	}
	i++;
	while (text[i]!='\0')
	{
		value[j]=text[i];
		i++;
		j++;
	}
	value[j]='\0';
}




	
/** The getter methid for the client config dir (ccd).
 * @return A pointer to the ccd.
 */
char * Config::getCcdPath(void)
{
	return this->ccdPath;
}


/** The setter method for the client config dir (ccd).
 * @param path A string to the ccd path.
 */
void Config::setCcdPath(char * path)
{
	strncpy(this->ccdPath,path, 256);
}

/** Returns the path to the status file.
 * @param A pointer to path of the status file.
 */
char * Config::getStatusFile(void)
{
	return this->statusFile;
}

/** The setter method for thepath to the statusfile (path + filename).
 * @param file A string of the filepath.
 */
void Config::setStatusFile(char *file)
{
	strncpy(this->statusFile,file, 256);
}

/** The setter method for the nas ip address.
 * @param ip A string with ip address.
 */	
void Config::setSubnet(char * ip)
{
	strncpy(this->subnet,ip, 16);
}


/** The getter method for the nas ip address.
 * @return A pointer to the nas ipaddress.
 */
char * Config::getSubnet(void)
{
	return this->subnet;
}

/** The setter method for the p2p address.
 * @param ip A string with p2p address.
 */	
void Config::setP2p(char * ip)
{
	strncpy(this->p2p,ip, 16);
}


/** The getter method for the p2p address.
 * @return A pointer to the p2p address.
 */
char * Config::getP2p(void)
{
	return this->p2p;
}

