/*
 *  radiusplugin -- An OpenVPN plugin for do radius authentication 
 *					and accounting.
 * 
 *  Copyright (C) 2005 EWE TEL GmbH/Ralf Luebben <ralfluebben@gmx.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
#include "User.h"

/** The constructor sets the acctinteriminterval and the portnumber to 0.*/
User::User()
{
	this->framedip="";
	this->framedroutes="";
	this->acctinteriminterval=0;
	this->portnumber=0;
}

/** The constructor sets the acctinteriminterval to 0 and the portnumber to num.
 * @param num  The portnumber.*/
User::User(int num)
{
	this->framedip="";
	this->framedroutes="";
	this->acctinteriminterval=0;
	this->portnumber=num;
}

/** The destructor. Nothing happens here.*/
User::~User()
{
}

/** The overloading of the assignment operator.
 * @param u A reference to an object of the User class.
 * @return A reference to the User class.*/
User & User::operator=(const User & u)
{
	this->username=u.username;
	this->commonname=u.commonname;
	this->framedroutes=u.framedroutes;
	this->framedip=u.framedip;
	this->callingstationid=u.callingstationid;
	this->portnumber=u.portnumber;
	this->acctinteriminterval=u.acctinteriminterval;
	return *this;
}

/** The copy constructor.
 * @param u A reference to an object of the User class.*/ 

User::User(const User & u)
{
	this->username=u.username;
	this->commonname=u.commonname;
	this->framedroutes=u.framedroutes;
	this->framedip=u.framedip;
	this->callingstationid=u.callingstationid;
	this->portnumber=u.portnumber;
	this->acctinteriminterval=u.acctinteriminterval;
}

/** The getter method for the username.
 * @return The username as a string.*/
string User::getUsername(void)
{
	return this->username;
}
/** The setter method for the username.
 * @param uname The username.*/
void User::setUsername(string uname)
{
	this->username=uname;
}

/** The getter method for the commonname.
 *  @return The commonname as a string.*/
string User::getCommonname(void)
{
	return this->commonname;
}
/** The setter method for the commonname.
 * @param cn The commonname.*/
void User::setCommonname(string cn)
{
	this->commonname=cn;
}

/** The getter method for the framed routes.
 *  @return The framed routes as a string.*/	
string User::getFramedRoutes(void)
{
	return this->framedroutes;
}
/** The setter method for the framedroutes.
 * @param froutes The framedroutes, if there are more 
 * routes they are diveded through a ';'.*/
void User::setFramedRoutes(string froutes)
{
	this->framedroutes=froutes;
}

/** The getter method for the framed ip.
 *  @return The framed ip as a string.*/
string User::getFramedIp(void)
{
	return this->framedip;
}
/** The setter method for the framedip.
 * @param ip The framedip.*/
void User::setFramedIp(string ip)
{
	this->framedip=ip;
}

/** The getter method for the calling station id.
 *  @return The calling station id as a string.*/
string User::getCallingStationId(void)
{
	return this->callingstationid;
}
/** The setter method for the callingstationid.
 * @param id The callingstationid.*/
void User::setCallingStationId(string id)
{
	this->callingstationid=id;
}

/** The getter method for the portnumber.
 *  @return The portnumber as an integer.*/
int User::getPortnumber(void)
{
	return this->portnumber;
}
/** The setter method for the portnumber.
 * @param port The portnumber.*/
void User::setPortnumber(int port)
{
	this->portnumber=port;
}

/** The getter method for the acctinteriminterval.
 *  @return The acctinteriminterval as struct time_t.*/
time_t User::getAcctInterimInterval(void)
{
	return this->acctinteriminterval;
}
/** The setter method for the username.
 * @param t  The acctinteriminterval.*/
void User::setAcctInterimInterval(time_t t)
{
	this->acctinteriminterval=t;
}
