/*
 *  radiusplugin -- An OpenVPN plugin for do radius authentication 
 *					and accounting.
 * 
 *  Copyright (C) 2005 EWE TEL GmbH/Ralf Luebben <ralfluebben@gmx.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
#include "UserAuth.h"



/** The constructor. Nothing happens here.*/
UserAuth::UserAuth():User()
{
  
}
/** The constructor. Nothing happens here.*/
UserAuth::~UserAuth()
{
}

/**The method send an authentication packet to the radius server and
 * calls the method parseResponsePacket(). The following attributes are in the packet:
 * - User_Name,
 * - User_Password
 * - NAS_PortCalling_Station_Id,
 * - NAS_Identifier,
 * - NAS_IP_Address, 
 * - NAS_Port_Type
 * - Service_Type.
 * @param context The context of the background process.
 * @return An integer, 0 if the authentication succeded, else 1.*/
int UserAuth::sendAcceptRequestPacket(PluginContext * context)
{
	list<RadiusServer> * serverlist;
	list<RadiusServer>::iterator server;
	RadiusPacket		packet(ACCESS_REQUEST);
	RadiusAttribute		ra1(ATTRIB_User_Name,this->getUsername().c_str()),
						ra2(ATTRIB_User_Password),
						ra3(ATTRIB_NAS_Port,this->getPortnumber()),
						ra4(ATTRIB_Calling_Station_Id,this->getCallingStationId()),
						ra5(ATTRIB_NAS_Identifier),
						ra6(ATTRIB_NAS_IP_Address),
						ra7(ATTRIB_NAS_Port_Type),
						ra8(ATTRIB_Service_Type);
	
	
	if (DEBUG (context->getVerbosity()))
    	fprintf (stderr, "RADIUS-PLUGIN: radius_server().\n");
		
	//get the server list
	serverlist=context->radiusconf.getRadiusServer();
	//set server to the first server
	server=serverlist->begin();
	
	
	if (DEBUG (context->getVerbosity()))
		cerr << "RADIUS-PLUGIN: Build password packet:  password: " << this->password << ", sharedSecret: " <<server->getSharedSecret() << " .\n";
	
	//add the attributes
	ra2.setValue(this->password);
	if(packet.addRadiusAttribute(&ra1))
	{
		fprintf(stderr, "RADIUS-PLUGIN: Fail to add attribute ATTRIB_User_Name.\n");
	}
	
	if (packet.addRadiusAttribute(&ra2))
	{
		fprintf(stderr, "RADIUS-PLUGIN: Fail to add attribute ATTRIB_User_Password.\n");
	}
	if (packet.addRadiusAttribute(&ra3))
	{
		fprintf(stderr, "RADIUS-PLUGIN: Fail to add attribute ATTRIB_NAS_Port.\n");
	}
	if (packet.addRadiusAttribute(&ra4))
	{
		fprintf(stderr, "RADIUS-PLUGIN: Fail to add attribute ATTRIB_Calling_Station_Id.\n");
	}
	//get information from the config and add it to the packet
	if(strcmp(context->radiusconf.getNASIdentifier(),""))
	{
			ra5.setValue(context->radiusconf.getNASIdentifier());
			if (packet.addRadiusAttribute(&ra5))
			{
				fprintf(stderr, "RADIUS-PLUGIN: Fail to add attribute ATTRIB_NAS_Identifier.\n");
			}
	}
	
	if(strcmp(context->radiusconf.getNASIpAddress(),""))
	{
			if(ra6.setValue(context->radiusconf.getNASIpAddress())!=0)
			{
				fprintf(stderr, "RADIUS-PLUGIN: Fail to set value ATTRIB_NAS_Ip_Address.\n");
			}
			else
			if (packet.addRadiusAttribute(&ra6))
			{
				fprintf(stderr, "RADIUS-PLUGIN: Fail to add attribute ATTRIB_NAS_Ip_Address.\n");
			}
	}
	if(strcmp(context->radiusconf.getNASPortType(),""))
	{
			ra7.setValue(context->radiusconf.getNASPortType());
			if (packet.addRadiusAttribute(&ra7))
			{
				fprintf(stderr, "RADIUS-PLUGIN: Fail to add attribute ATTRIB_NAS_Port_Type.\n");
			}
	}
	
	if(strcmp(context->radiusconf.getServiceType(),""))
	{
			ra8.setValue(context->radiusconf.getServiceType());
			if (packet.addRadiusAttribute(&ra8))
			{
				fprintf(stderr, "RADIUS-PLUGIN: Fail to add attribute ATTRIB_Service_Type.\n");
			}
	}
	
	
	if (DEBUG (context->getVerbosity()))
		fprintf (stderr, "RADIUS-PLUGIN: Send packet to %s.\n",server->getName().c_str());
	//send the packet
	if (packet.radiusSend(server)<0)
	{
		fprintf(stderr, "RADIUS-PLUGIN: Packet was not send.\n");
	}
	//receive the packet
	if (packet.radiusReceive(serverlist)==0)
	{
		//is it a accept?
		if(packet.getCode()==ACCESS_ACCEPT)
		{
			if (DEBUG (context->getVerbosity()))
				fprintf (stderr, "RADIUS-PLUGIN: Get ACCESS_ACCEPT-Packet.\n");

			//parse the attributes for framedip, framedroutes and
			//acctinteriminterval
			this->parseResponsePacket(&packet, context);
			return 0;
			
		}
		else
		{
			fprintf (stderr, "RADIUS-PLUGIN: Get ACCESS_REJECT or ACCESS_CHALLANGE-Packet.->ACCESS-DENIED.\n");
			return 1;
		}
		
	}
	else
	{
		fprintf (stderr, "RADIUS-PLUGIN: Got no response from radius server.\n");
	}
	
	return 1;
		
}

/** The method parse the authentication response packet for
 * the attributes framed ip, framed routes and accinteriminterval 
 * and saves the values in the UserAuth object. The there is no acctinteriminterval
 * it is set to 0.
 * @param packet A pointer to the radius packet to parse.
 * @param context The plugin context.
 */

void UserAuth::parseResponsePacket(RadiusPacket *packet, PluginContext * context)
{
	pair <multimap<Octet,RadiusAttribute>::iterator,multimap<Octet,RadiusAttribute>::iterator> range;
	multimap<Octet,RadiusAttribute>::iterator iter1, iter2;
	
		
	if (DEBUG (context->getVerbosity()))
    	fprintf (stderr, "RADIUS-PLUGIN: parse_response_packet().\n");
	
	
	
	range=packet->findAttributes(22);
	iter1=range.first;
	iter2=range.second;	
	string froutes;
	while (iter1!=iter2)
	{
		
		froutes.append((char *) iter1->second.getValue(),iter1->second.getLength()-2);
		froutes.append(";");
		iter1++;
	
	}
	this->setFramedRoutes(froutes);
	
	
	if (DEBUG (context->getVerbosity()))
    	cerr << "RADIUS-PLUGIN: BACKGROUND ACCT: routes: " << this->getFramedRoutes() <<".\n";
	
	
	range=packet->findAttributes(8);
	iter1=range.first;
	iter2=range.second;	
	
	
	if (iter1!=iter2)
	{
		this->setFramedIp(iter1->second.ipFromBuf());
	}
	
	if (DEBUG (context->getVerbosity()))
    	cerr << "RADIUS-PLUGIN: BACKGROUND ACCT: framed ip: " << this->getFramedIp() <<".\n";
	
	
	
	range=packet->findAttributes(85);
	iter1=range.first;
	iter2=range.second;		
	if (iter1!=iter2)
	{
		this->setAcctInterimInterval(iter1->second.intFromBuf());
	}
	else
	{
		fprintf(stderr,"RADIUS-PLUGIN: No attributes Acct Interim Interval or bad length.\n");
	}
	
	if (DEBUG (context->getVerbosity()))
    	cerr << "RADIUS-PLUGIN: BACKGROUND ACCT: Acct Interim Interval: " << this->getAcctInterimInterval() << ".\n";
	
}

/** The method creates the client config file in the client config dir (ccd).
 * The path is set in the radiusplugin config file.
 * @param context : The plugin context.
 * @return An integer, if everything is ok 0, else 1.
 */
int UserAuth::createCcdFile(PluginContext *context)
{
	ofstream ccdfile;
	
	char * route;
	char framedip[16];
	char ipstring[100];
	in_addr_t ip2;
	in_addr ip3;
	char filename[512];
	char framedroutes[4096];
	char framednetmask_cidr[3]; // ->/24
	char framednetmask[16]; // ->255.255.255.0
	char mask_part[6];
	char framedgw[16];
	char framedmetric[5]; //what is the biggest metric? 
	
	double d1,d2;
	
	int j=0,k=0;
	int len=0;
	
	
	
	memset(filename,0,512);
	memset(ipstring,0,100);
	memset(framedip,0,16);
	memset(framedroutes,0,4096);
	
	
	//create the filename, ccd-path + commonname
	strncat(filename,context->conf.getCcdPath(),255);
	strncat(filename,this->getCommonname().c_str(),127);
	
	if (DEBUG (context->getVerbosity()))
    	fprintf (stderr, "RADIUS-PLUGIN: BACKGROUND AUTH: Try to open ccd file.\n");
	
	//open the file
	ccdfile.open(filename,ios::out);
	
	if (DEBUG (context->getVerbosity()))
    	fprintf (stderr, "RADIUS-PLUGIN: BACKGROUND AUTH: Opened ccd file.\n");
	
	
	// copy in a temp-string, becaue strtok deletes the delimiter, if it used anywhere
	strncpy(framedroutes,this->getFramedRoutes().c_str(),4095);
	
	
	if (ccdfile.good())
	{
		
		//set the ip address in the file
		if (this->framedip[0]!='\0')
		{
			if (DEBUG (context->getVerbosity()))
				fprintf (stderr, "RADIUS-PLUGIN: BACKGORUND AUTH: Write framed ip to ccd-file.\n");
		
			//build the ifconfig
			strncat(ipstring, "ifconfig-push ",14);
			strncat(ipstring, this->getFramedIp().c_str() , 15);
			strncat(ipstring, " ", 1);
			
			
			if(context->conf.getSubnet()[0]!='\0')
			{
				strncat(ipstring, context->conf.getSubnet() , 15);
				if (DEBUG (context->getVerbosity()))
					fprintf (stderr, "RADIUS-PLUGIN: BACKGORUND AUTH: Create ifconfig-push for topology subnet.\n");
		
			}
			else if(context->conf.getP2p()[0]!='\0')
			{
				strncat(ipstring, context->conf.getP2p() , 15);
				if (DEBUG (context->getVerbosity()))
					fprintf (stderr, "RADIUS-PLUGIN: BACKGORUND AUTH: Create ifconfig-push for topology p2p.\n");
		
			}
			else
			{
				//increment the last byte of the ip address
				//in interface needs two addresses because it is a 
				//convert from string to integer in network byte order
				ip2=inet_addr(this->getFramedIp().c_str());
				//convert from network byte order to host byte order
				ip2=ntohl(ip2);
				//increment
				ip2++;
				//convert from host byte order to network byte order
				ip2=htonl(ip2);
				//copy from one unsigned int to another (casting don't work with these struct!?)
				memcpy(&ip3, &ip2, 4);
				// append the new ip address to the string
				strncat(ipstring, inet_ntoa(ip3), 15);
				if (DEBUG (context->getVerbosity()))
					fprintf (stderr, "RADIUS-PLUGIN: BACKGORUND AUTH: Create ifconfig-push for topology net30.\n");
		
			}
			if (DEBUG (context->getVerbosity()))
				fprintf (stderr, "RADIUS-PLUGIN: Write %s ccd-file.\n",ipstring);
			
			
			ccdfile << ipstring <<"\n";
		}
		
		//set the framed routes in the file for the openvpn process
		if (framedroutes[0]!='\0')
		{
			if (DEBUG (context->getVerbosity()))
				fprintf (stderr, "RADIUS-PLUGIN: BACKGORUND AUTH: Write framed routes to ccd-file.\n");
		
			route=strtok(framedroutes,";");
			len=strlen(route);
			if (len > 50) //this is to big!! but the length is variable
			{
				fprintf(stderr,"RADIUS-PLUGIN: Argument for Framed Route is to long (>100 Characters).\n");
				return 1;
			}
			else
			{
				while (route!=NULL)
				{
					j=0;k=0;
					//set everything back for the next route entry
					memset(mask_part,0,6);
					memset(framednetmask_cidr,0,3);
					memset(framedip,0,16);
					memset(framednetmask,0,16);
					memset(framedgw,0,16);
					memset(framedmetric,0,5);
					
					//add ip address to string
					while(route[j]!='/' && j<len)
						{
							if (route[j]!=' ')
							{
								framedip[k]=route[j];
								k++;
							}
							j++;
						}
						k=0;
						j++;
						//add netmask
						while(route[j]!=' ' && j<=len)
						{
							framednetmask_cidr[k]=route[j];
							k++;
							j++;
						}
						k=0;
						//jump spaces
						while(route[j]==' ' && j<len)
						{
							j++;
						}
						//find gateway
						while(route[j]!='/' && j<len)
						{
							if (route[j]!=' ')
							{
								framedgw[k]=route[j];
								k++;
							}
							j++;
						}
						j++;
						
						//find gateway netmask (this isn't used
						//at the command route under linux)
						while(route[j]!=' ' && j<len)
						{
							j++;
						}
						//jump spaces
						
						while(route[j]==' ' && j<len )
						{
							j++;
						}
						k=0;
						if (j<=len)
						{
						
							k=0;
							//find the metric
							while(route[j]!=' ' && j<len)
							{
								framedmetric[k]=route[j];
								k++;
								j++;
							}
						}
																							
						//create string for client config file
						//transform framednetmask_cidr
						d2=7;
						d1=0;
						memset(framednetmask,0,16);
						if (atoi(framednetmask_cidr)>32)
						{
							fprintf(stderr, "RADIUS-PLUGIN: Bad net CIDR netmask.\n");
						}
						else
						{
							for (k=1; k<=atoi(framednetmask_cidr); k++)
							{
								d1=d1+pow(2,d2);
								d2--;
								
								if (k==8)
								{
									sprintf(mask_part,"%.0lf.", d1);
									d1=0;
									d2=7;
									strncat(framednetmask, mask_part, 4);
									memset(mask_part,0,6);
								}
								if(k==16)
								{
									sprintf(mask_part,"%.0lf.", d1);
									d1=0;
									d2=7;
									strncat(framednetmask, mask_part, 4);
									memset(mask_part,0,6);
								}
								if(k==24)
								{
									sprintf(mask_part,"%.0lf.", d1);
									d1=0;
									d2=7;
									strncat(framednetmask, mask_part, 4);
									memset(mask_part,0,6);
								}
							}
							if (j<8)
							{
									sprintf(mask_part,"%.0lf.", d1);
									d1=0;
									strncat(framednetmask, mask_part, 4);
									strncat(framednetmask, "0.0.0", 5);
									memset(mask_part,0,6);
							}
							else if (j<16)
							{
									sprintf(mask_part,"%.0lf.", d1);
									d1=0;
									strncat(framednetmask, mask_part, 4);
									strncat(framednetmask, "0.0", 3);
									memset(mask_part,0,6);
							}
							else if (j<24)
							{
									sprintf(mask_part,"%.0lf.", d1);
									d1=0;
									strncat(framednetmask, mask_part, 4);
									strncat(framednetmask, "0", 1);
									memset(mask_part,0,6);
							}
							else if (j>24)
							{
									sprintf(mask_part,"%.0lf", d1);
									d1=0;
									strncat(framednetmask, mask_part, 4);
									memset(mask_part,0,6);
							}
							
							
						}
						
						if (DEBUG (context->getVerbosity()))
	    						fprintf (stderr, "RADIUS-PLUGIN: Write route string: %s %s %s to ccd-file.\n","iroute " ,framedip, framednetmask);
		
						//write iroute to client file
						ccdfile << "iroute " << framedip << " "<< framednetmask << "\n";
					
						route=strtok(NULL,";");
				}
			}
		}
	
	
	}
	else
	{
		fprintf(stderr, "RADIUS-PLUGIN: Could not open file %s.\n",filename );
		return 1;
	}
	ccdfile.close();
	return 0;
}

/** The getter method for the password.
 * @return The password as a string.
 */
string UserAuth::getPassword(void)
{
	return this->password;
}
/**The setter method for the password.
 * @param passwd The password.
 */
void UserAuth::setPassword(string passwd)
{
	this->password=passwd;
}
